#if defined _SWGM_included
 #endinput
#endif
#define _SWGM_included

enum Status
{
	UNASSIGNED = 0,
	NO_GROUP,
	LEAVER,
	MEMBER,
	OFFICER
};

/**
 *	Вызывается когда игрок был проверен на группу. 
 *
 *	@param iClient			Индекс игрока.
 *	@param IsMember			Игрок в группе ?
 *	@param IsOfficer		Игрок Админ/Офицер в группе ?
 *	@noreturn
*/
forward void SWGM_OnJoinGroup(int iClient, bool IsMember, bool IsOfficer);

/**
 *	Вызывается когда игрок повторно проверяется и он вышел из группы. 
 *
 *	@param iClient			Индекс игрока.
 *	@noreturn
*/
forward void SWGM_OnLeaveGroup(int iClient);

/**
 *  Вызывается когда игрок проверяется на состояние в группе.
 *
 *  @param iClient          Индекс игрока.
 *  @return                 true чтобы разрешить, false чтобы заблокировать.
 */
forward bool SWGM_OnGroupCheck(int iClient);

/**
 *  Проверяет группу игрока.
 *
 *  @param iClient          Индекс игрока.
 *  @return                 true в случае успеха, иначе false.
 *  @error                  Игрок не на сервере, является ботом или его индекс превышает допустимую границу.
 */
native bool SWGM_InGroup(int iClient);

/**
 *  Проверяет на Администратора или Офицера группы.
 *
 *  @param iClient          Индекс игрока.
 *  @return                 true в случае успеха, иначе false.
 *  @error                  Игрок не на сервере, является ботом или его индекс превышает допустимую границу.
 */
native bool SWGM_InGroupOfficer(int iClient);

/**
 *  Возвращает статус игрока.
 *
 *  @param iClient          Индекс игрока.
 *  @return                 Статус.
 *  @error                  Игрок не на сервере, является ботом или его индекс превышает допустимую границу.
 */
native Status SWGM_GetPlayerStatus(int iClient);

/**
 *  Запускает проверку игрока на состояние в группе.
 *
 *  @param iClient          Индекс игрока.
 *  @error                  Игрок не на сервере, является ботом или его индекс превышает допустимую границу.
 *  @noreturn
 */
native void SWGM_CheckPlayer(int iClient);


public SharedPlugin __pl_SWGM= 
{
	name = "SWGM",
	file = "SWGM.smx",
#if defined REQUIRE_PLUGIN
	required = 1
#else
	required = 0
#endif
};

#if !defined REQUIRE_PLUGIN
public __pl_SWGM_SetNTVOptional()
{
	MarkNativeAsOptional("SWGM_InGroup");
	MarkNativeAsOptional("SWGM_InGroupOfficer");
	MarkNativeAsOptional("SWGM_GetPlayerStatus");
	MarkNativeAsOptional("SWGM_CheckPlayer");
}
#endif